/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include <sstream>
#include "EyeDoctor.h"
#include "DoctorView.h"
#include "SelectionState.h"
#include "ExperimentData.h"
#include ".\doctorview.h"

#define USE_MULTI_CHANGE 1

IMPLEMENT_DYNCREATE(CDoctorView, CFormView)

CDoctorView::CDoctorView()
	: CFormView(CDoctorView::IDD), _state(0)
{}

CDoctorView::~CDoctorView()
{}

void CDoctorView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CDoctorView, CFormView)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_DV_FIX_END, OnChangeFixEnd)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_DV_FIX_START, OnChangeFixStart)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_DV_FIX_XPIXEL, OnChangeFixXPixel)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_DV_FIX_YPIXEL, OnChangeFixYPixel)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_DV_TRIAL_SEQ, OnChangeTrialSeq)
	ON_WM_KEYDOWN()
	ON_STN_CLICKED(IDC_STATIC_DV_TRIAL_DC_TIME, OnStnClickedStaticDvTrialDcTime)
	ON_STN_CLICKED(IDC_STATIC_DV_TRIAL_DC_LEAD_SACC, OnStnClickedStaticDvTrialDcLeadSacc)
END_MESSAGE_MAP()

#ifdef _DEBUG
void CDoctorView::AssertValid() const
{
	CFormView::AssertValid();
}

void CDoctorView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif //_DEBUG

void CDoctorView::Initialize(int width, int height, int nTrials, int duration)
{
	((CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_DV_FIX_END))->SetRange(0, duration);
	((CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_DV_FIX_START))->SetRange(0, duration);
	((CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_DV_FIX_XPIXEL))->SetRange(0, width);
	((CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_DV_FIX_YPIXEL))->SetRange(0, height);
	((CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_DV_TRIAL_SEQ))->SetRange(0, nTrials-1);

}

void CDoctorView::OnChangeFixEnd(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);

	if(_state->hasExperiment()) {
		const SelectionList &selected = _state->selected();

		for(constSelectionIter i = selected.begin(); i != selected.end(); ++i) {
			EyeEvent *event = *i;
			try {
				event->update(0, -pNMUpDown->iDelta);
			} catch(std::exception &e) {
				MessageBox(e.what(), "Data Manipulation Error",  MB_ICONEXCLAMATION | MB_OK);
			}
		}
		theApp.UpdateAllViews();
	}

	*pResult = 0;
}

void CDoctorView::OnChangeFixStart(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);

	if(_state->hasExperiment()) {
		const SelectionList &selected = _state->selected();

		for(constSelectionIter i = selected.begin(); i != selected.end(); ++i) {
			EyeEvent *event = *i;
			try {
				event->update(-pNMUpDown->iDelta, 0);
			} catch(std::exception &e) {
				MessageBox(e.what(), "Data Manipulation Error",  MB_ICONEXCLAMATION | MB_OK);
			}
		}
		theApp.UpdateAllViews();
	}

	*pResult = 0;
}

void CDoctorView::OnChangeFixXPixel(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);

	if(_state->hasExperiment()) {
		const SelectionList &selected = _state->selected();
		TrialData *trial = _state->trial();

		for(constSelectionIter i = selected.begin(); i != selected.end(); ++i) {
			FixationEvent *event = *i;
			event->update(pNMUpDown->iDelta, 0);
			event->update(trial->getRegionLocation(event->pixelLocation()));
		}
		theApp.UpdateAllViews();
	}

	*pResult = 0;
}

void CDoctorView::OnChangeFixYPixel(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);

	if(_state->hasExperiment()) {
		const SelectionList &selected = _state->selected();
		TrialData *trial = _state->trial();

		for(constSelectionIter i = selected.begin(); i != selected.end(); ++i) {
			FixationEvent *event = *i;
			event->update(0, -pNMUpDown->iDelta);
			event->update(trial->getRegionLocation(event->pixelLocation()));
		}
		theApp.UpdateAllViews();
	}

	*pResult = 0;
}

void CDoctorView::OnChangeTrialSeq(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);

	if(_state->hasExperiment()) {
		_state->moveTrialPtr(pNMUpDown->iDelta);
		//_state->initSelectFirst();
		theApp.UpdateAllViews();	
	}
	*pResult = 0;
}

void CDoctorView::SetSelectionState(SelectionState *state)
{
	_state = state;
}

void CDoctorView::UpdateView()
{
	TrialData *trial = _state->trial();
	FixationEvent *event = _state->currFixation();

	const SaccadeEvent *lead = event ? event->leadSaccade() : 0;
	const SaccadeEvent *trail = event ? event->trailSaccade() : 0;
	const int maxlen = 20;
	char buf[maxlen];

	if(trial) {
		

			
		std::string status;

		GetDlgItem(IDC_STATIC_DV_TRIAL_SEQ)->SetWindowText(_itoa(trial->index(), buf, 10));
		GetDlgItem(IDC_STATIC_DV_TRIALID)->SetWindowText(trial->trialID().idString().c_str());
 		GetDlgItem(IDC_STATIC_DV_TRIAL_DURATION)->SetWindowText(_itoa(trial->duration(), buf, 10));

		unsigned int eTime;
		dcData d;
		if(trial->dcTime()){
			//GetDlgItem(IDC_STATIC_DV_TRIAL_DC_TIME)->SetWindowText(_itoa(trial->dcTime(), buf, 10));
			CPoint ePoint = event ? event->pixelLocation() : CPoint(0,0);
			/*unsigned int */eTime = event ? event->endTime() : 0;
			if(_state->useMultiChange()){
				d=trial->getDCData(eTime);
				//GetDlgItem(IDC_STATIC_DV_TRIAL_DC_TIME)->SetWindowText(_itoa(trial->getLastDCTime(eTime), buf, 10));
				if(d.time > 0){
					GetDlgItem(IDC_STATIC_DV_TRIAL_DC_TIME)->SetWindowText(_itoa(d.time,buf,10));	
					if(d.time >= d.leadEnd){
						GetDlgItem(IDC_STATIC_DV_TRIAL_DC_OK)->SetWindowText("LATE");
					}
					else 
						GetDlgItem(IDC_STATIC_DV_TRIAL_DC_OK)->SetWindowText("OK");
				}
				else {
					GetDlgItem(IDC_STATIC_DV_TRIAL_DC_TIME)->SetWindowText("");
					GetDlgItem(IDC_STATIC_DV_TRIAL_DC_OK)->SetWindowText("");
				}
			}

			else{
				GetDlgItem(IDC_STATIC_DV_TRIAL_DC_TIME)->SetWindowText(_itoa(trial->dcTime(), buf, 10));
			}
		}
		else 
			GetDlgItem(IDC_STATIC_DV_TRIAL_DC_TIME)->SetWindowText("");

		if(trial->dcTime()){
			if(_state->useMultiChange()){
				std::ostringstream sacc;
				if(d.time > 0){
					sacc<<_itoa((int)d.leadStart,buf,10);
					sacc<<"--";
					sacc<<_itoa((int)d.leadEnd,buf,10);
				}
				else {
					sacc<<"";
					//sacc<<"--";
					//sacc<<_itoa((int)d.leadEnd,buf,10);
				}
				//NEW 
				/*if(lead){
					sacc<<_itoa((int)trial->getLastLeadStart(eTime),buf,10);
					sacc<<"-";
					sacc<<_itoa((int)trial->getLastLeadEnd(eTime),buf,10);
					//sacc<<_itoa(lead->endTime(),buf,10);
					GetDlgItem(IDC_STATIC_DV_TRIAL_DC_LEAD_SACC)->SetWindowText(sacc.str().c_str());//_itoa(trial->dcLeadSaccStart,buf, 10));
				 }*/
				//if(lead && (lead->startTime() < trial->getLastDCTime(eTime) && lead->endTime() > trial->getLastDCTime(eTime))){
				//if(lead /*&& lead->startTime() < trial->getLastDCTime(eTime)*/) sacc<<_itoa(lead->startTime(),buf,10);
				//if(lead && lead->startTime() < trial->getLastDCTime(eTime)  && lead->endTime() > trial->getLastDCTime(eTime)) sacc<<_itoa((int)trial->getLastLeadStart(eTime),buf,10);
				//sacc<<"-";
				//if(lead /*&& lead->endTime() < trial->getLastDCTime(eTime)*/) sacc<<_itoa(lead->endTime(), buf,10);
				GetDlgItem(IDC_STATIC_DV_TRIAL_DC_LEAD_SACC)->SetWindowText(sacc.str().c_str());//_itoa(trial->dcLeadSaccStart,buf, 10));
				 //}
			}
			else{
				std::ostringstream sacc;
				sacc<<_itoa(trial->dcLeadSaccStart,buf,10);
				sacc<<"-";
				sacc<<_itoa(trial->dcLeadSaccEnd, buf,10);
				GetDlgItem(IDC_STATIC_DV_TRIAL_DC_LEAD_SACC)->SetWindowText(sacc.str().c_str());//_itoa(trial->dcLeadSaccStart,buf, 10));
			}

		}
		else
			GetDlgItem(IDC_STATIC_DV_TRIAL_DC_LEAD_SACC)->SetWindowText("");

		if(trial->testFlag(TRIAL_DELETED))
			status = "DELETED";
		else if(trial->testFlag(TRIAL_REPEATED))
			status = "REPEATED";
		else if(trial->testFlag(TRIAL_ABORTED))
			status = "ABORTED";
		else if(trial->testFlag(TRIAL_ERROR))
			status = "ERROR";
		else if(trial->testFlag(TRIAL_TIMEOUT))
			status = "TIMED OUT";
		else if(trial->testFlag(J_HOOK_DISP_CHANGE))
			status = "J HOOK";
		else if(trial->testFlag(BLINK_CHANGE_ERROR)) // change happened during blink (ie, data points recorded during blink suggesting tracking error)
			status = "ERROR";
		else status = "OKAY";
		GetDlgItem(IDC_STATIC_DV_TRIAL_STATUS)->SetWindowText(status.c_str());
	}
	else {
		GetDlgItem(IDC_STATIC_DV_TRIAL_SEQ)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_TRIALID)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_TRIAL_DURATION)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_TRIAL_STATUS)->SetWindowText("");
	}

	if(event) {
		CPoint ch = event->charLocation();
		CPoint pixel = event->pixelLocation();
		int i = 0;

		_itoa(event->index(), buf, 10);          //construct fixation index
		while(i < maxlen && buf[i] != '\0') ++i;
		buf[i++] = ' ';
		buf[i++] = event->isRightEye() ? 'R' : 'L';
		buf[i++] = '\0';

		GetDlgItem(IDC_STATIC_DV_FIX_INDEX)->SetWindowText(buf);
		GetDlgItem(IDC_STATIC_DV_FIX_XPIXEL)->SetWindowText(_itoa(pixel.x, buf, 10));
		GetDlgItem(IDC_STATIC_DV_FIX_YPIXEL)->SetWindowText(_itoa(pixel.y, buf, 10));
		GetDlgItem(IDC_STATIC_DV_FIX_XCHAR)->SetWindowText(_itoa(ch.x, buf, 10));
		GetDlgItem(IDC_STATIC_DV_FIX_YCHAR)->SetWindowText(_itoa(ch.y, buf, 10));
		GetDlgItem(IDC_STATIC_DV_FIX_START)->SetWindowText(_itoa(event->startTime(), buf, 10));
		GetDlgItem(IDC_STATIC_DV_FIX_END)->SetWindowText(_itoa(event->endTime(), buf, 10));
		GetDlgItem(IDC_STATIC_DV_FIX_DURATION)->SetWindowText(_itoa(event->duration(), buf, 10));

		((CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_DV_FIX_XPIXEL))->SetPos(pixel.x);
		((CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_DV_FIX_YPIXEL))->SetPos(pixel.y);
	}
	else {
		GetDlgItem(IDC_STATIC_DV_FIX_INDEX)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_FIX_XPIXEL)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_FIX_YPIXEL)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_FIX_XCHAR)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_FIX_YCHAR)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_FIX_START)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_FIX_END)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_FIX_DURATION)->SetWindowText("");
	}

	if(lead) {
		std::string endLoc(_itoa(lead->endPixel().x, buf, 10));
		std::string startLoc(_itoa(lead->startPixel().x, buf, 10));
	
		endLoc += ", ";
		endLoc += _itoa(lead->endPixel().y, buf, 10);
		startLoc += ", ";
		startLoc += _itoa(lead->startPixel().y, buf, 10);

		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_START_TIME)->SetWindowText(_itoa(lead->startTime(), buf, 10));
		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_END_TIME)->SetWindowText(_itoa(lead->endTime(), buf, 10));
		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_DURATION)->SetWindowText(_itoa(lead->duration(), buf, 10));
		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_START_LOC)->SetWindowText(startLoc.c_str());
		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_END_LOC)->SetWindowText(endLoc.c_str());
	}
	else {
		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_START_TIME)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_END_TIME)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_DURATION)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_START_LOC)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_LEAD_SACC_END_LOC)->SetWindowText("");
	}

	if(trail) {
		std::string endLoc(_itoa(trail->endPixel().x, buf, 10));
		std::string startLoc(_itoa(trail->startPixel().x, buf, 10));
	
		endLoc += ", ";
		endLoc += _itoa(trail->endPixel().y, buf, 10);
		startLoc += ", ";
		startLoc += _itoa(trail->startPixel().y, buf, 10);

		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_START_TIME)->SetWindowText(_itoa(trail->startTime(), buf, 10));
		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_END_TIME)->SetWindowText(_itoa(trail->endTime(), buf, 10));
		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_DURATION)->SetWindowText(_itoa(trail->duration(), buf, 10));
		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_START_LOC)->SetWindowText(startLoc.c_str());
		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_END_LOC)->SetWindowText(endLoc.c_str());
	}
	else {
		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_START_TIME)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_END_TIME)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_DURATION)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_START_LOC)->SetWindowText("");
		GetDlgItem(IDC_STATIC_DV_TRAIL_SACC_END_LOC)->SetWindowText("");
	}
}

void CDoctorView::OnStnClickedStaticDvTrialDcTime()
{
	// TODO: Add your control notification handler code here
}

void CDoctorView::OnStnClickedStaticDvTrialDcLeadSacc()
{
	// TODO: Add your control notification handler code here
}
