/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#pragma warning( disable : 4290 )

#include <exception>
#include <iosfwd>
#include "Selections.h"

class EyeEvent {
public:
	EyeEvent();
	EyeEvent(bool rightEye, bool trackLoss, bool boundary, unsigned int start, unsigned int end);
	EyeEvent(const EyeEvent &event);
	virtual ~EyeEvent();

	const EyeEvent& operator=(const EyeEvent &rhs);

	EyeEvent *next(bool ignoreDeleted = true) const;
	EyeEvent *previous(bool ignoreDeleted = true) const;

	unsigned int duration() const;
	unsigned int endTime() const;
	unsigned int startTime() const;

	int length() const;

	bool isDeleted() const;
	bool isRightEye() const;
	bool isTrackLoss() const;

	void deleteEvent();
	void setNext(EyeEvent *event);
	void setPrevious(EyeEvent *event);
	void truncateFutureEvents();
	void update(int deltaStartTime, int deltaEndTime) throw(std::exception);
	void updateInitial(int deltaStartTime, int deltaEndTime) throw(std::exception);

	virtual void draw(CDC *pDC, const Colors &colors, const StateFlags &flags);
	virtual void export(std::ostream &os) const;
	virtual void read(std::istream &is, int versionNum);
	virtual void revert();
	virtual void write(std::ostream &os) const;

protected:

	struct EyeData {
		EyeData();
	    const EyeData &operator=(const EyeData &rhs);

		unsigned int startTime;
		unsigned int endTime;
		bool         isBoundary;
		bool         isDeleted;
		bool         isRight;
		bool         isTrackLoss;

		bool		crossesDCboundary;
	};

	EyeData   _ed;
	EyeData   _edBackup;
	EyeEvent *_next;
	EyeEvent *_prev;
};
