/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeDoctor.h"
#include "FinalFixDialog.h"
#include ".\finalfixdialog.h"


// CFinalFixDialog dialog

IMPLEMENT_DYNAMIC(CFinalFixDialog, CDialog)

CFinalFixDialog::CFinalFixDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CFinalFixDialog::IDD, pParent), _deleteLen(40), 
	  _mergeLen(80), _deleteDist(3), _mergeDist(1), 
	  _exportDep(false), _left(false), _right(false),
	  _excludeZeros(false), _excludeFives(false), _excludeTens(false),
	  _excludeJHooks(false), _excludeBlinks(false), _excludeLongSacc(false)
{}

CFinalFixDialog::~CFinalFixDialog()
{}

void CFinalFixDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CFinalFixDialog, CDialog)
	ON_EN_CHANGE(IDC_EDIT_FINAL_FIX_DIST1, OnEnChangeEditFinalFixDist1)
	ON_BN_CLICKED(IDC_CHECK_FINAL_INCLUDE_DEP, OnBnClickedCheckFinalIncludeDep)
	ON_BN_CLICKED(IDC_CHECK_EXCLUDE_DC_10, OnBnClickedCheckExcludeDc10)
	ON_BN_CLICKED(IDC_CHECK_EXCLUDE_DC_BLINK, OnBnClickedCheckExcludeDcBlink)
	ON_BN_CLICKED(IDC_CHECK_EXCLUDE_LONG_SACC, OnBnClickedCheckExcludeLongSacc)
END_MESSAGE_MAP()

int	CFinalFixDialog::DeleteFixDist() const
{
	return(_deleteDist);
}

unsigned int CFinalFixDialog::DeleteFixLen() const
{
	return(_deleteLen);
}

bool CFinalFixDialog::ExportDependents() const
{
	return(_exportDep);
}


bool CFinalFixDialog::ExcludeTens() const
{
	return(_excludeTens);
}

bool CFinalFixDialog::ExcludeFives() const
{
	return(_excludeFives);
}

bool CFinalFixDialog::ExcludeZeros() const
{
	return(_excludeZeros);
}

bool CFinalFixDialog::ExcludeJHooks() const
{
	return(_excludeJHooks);
}

bool CFinalFixDialog::ExcludeBlinks() const
{
	return(_excludeBlinks);
}

bool CFinalFixDialog::ExcludeLongSacc() const
{
	return(_excludeLongSacc);
}

int CFinalFixDialog::MergeFixDist() const
{
	return(_mergeDist);
}

unsigned int CFinalFixDialog::MergeFixLen() const
{
	return(_mergeLen);
}

BOOL CFinalFixDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	GetDlgItem(IDC_EDIT_FINAL_FIX_DIST1)->SetWindowText("1");
	GetDlgItem(IDC_EDIT_FINAL_FIX_DIST2)->SetWindowText("3");
	GetDlgItem(IDC_EDIT_FINAL_FIX_LEN1)->SetWindowText("80");
	GetDlgItem(IDC_EDIT_FINAL_FIX_LEN2)->SetWindowText("40");

	if(_left)
		((CButton*)GetDlgItem(IDC_RADIO_FINAL_LEFT_EYE))->SetCheck(BST_CHECKED);
	else GetDlgItem(IDC_RADIO_FINAL_LEFT_EYE)->EnableWindow(FALSE);

	if(_right) {
		if(!_left)
			((CButton*)GetDlgItem(IDC_RADIO_FINAL_RIGHT_EYE))->SetCheck(BST_CHECKED);
	}
	else GetDlgItem(IDC_RADIO_FINAL_RIGHT_EYE)->EnableWindow(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CFinalFixDialog::OnOK()
{
	const int max = 20;
	int state;
	char buf[max];

	GetDlgItem(IDC_EDIT_FINAL_FIX_DIST1)->GetWindowText(buf, max);
	_mergeDist = atoi(buf);

	GetDlgItem(IDC_EDIT_FINAL_FIX_DIST2)->GetWindowText(buf, max);
	_deleteDist = atoi(buf);

	GetDlgItem(IDC_EDIT_FINAL_FIX_LEN1)->GetWindowText(buf, max);
	_mergeLen = atoi(buf);

	GetDlgItem(IDC_EDIT_FINAL_FIX_LEN2)->GetWindowText(buf, max);
	_deleteLen = atoi(buf);

	state = ((CButton*)GetDlgItem(IDC_CHECK_FINAL_INCLUDE_DEP))->GetCheck();
	_exportDep = (state == BST_CHECKED);

	state = ((CButton*)GetDlgItem(IDC_CHECK_EXCLUDE_DC_10))->GetCheck();
	_excludeTens = (state == BST_CHECKED);
	state = ((CButton*)GetDlgItem(IDC_CHECK_EXCLUDE_DC_5))->GetCheck();
	_excludeFives = (state == BST_CHECKED);
	state = ((CButton*)GetDlgItem(IDC_CHECK_EXCLUDE_DC_ALL))->GetCheck();
	_excludeZeros = (state == BST_CHECKED);
	state = ((CButton*)GetDlgItem(IDC_CHECK_EXCLUDE_DC_JHOOK))->GetCheck();
	_excludeJHooks = (state == BST_CHECKED);
	state = ((CButton*)GetDlgItem(IDC_CHECK_EXCLUDE_DC_BLINK))->GetCheck();
	_excludeBlinks = (state == BST_CHECKED);
	state = ((CButton*)GetDlgItem(IDC_CHECK_EXCLUDE_LONG_SACC))->GetCheck();
	_excludeLongSacc = (state == BST_CHECKED);

	if(_mergeLen < _deleteLen) {
		MessageBox("Delete duration must be less than combine duration.", "Parameter Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}

	if(_mergeDist > _deleteDist) {
		MessageBox("Combine distance must be less than delete distance.", "Parameter Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}

	state = ((CButton*)GetDlgItem(IDC_RADIO_FINAL_LEFT_EYE))->GetCheck();
	_left = (state == BST_CHECKED);

	state = ((CButton*)GetDlgItem(IDC_RADIO_FINAL_RIGHT_EYE))->GetCheck();
	_right = (state == BST_CHECKED);

	if(!_left && !_right) {
		MessageBox("Select source eye for data.", "Parameter Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}

	CDialog::OnOK();
}

void CFinalFixDialog::SetAvailableEyeData(bool left, bool right)
{
	_left = left;
	_right = right;
}

bool CFinalFixDialog::UseRightEye() const
{
	return(_right);
}

void CFinalFixDialog::OnEnChangeEditFinalFixDist1()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void CFinalFixDialog::OnBnClickedCheckFinalIncludeDep()
{
	// TODO: Add your control notification handler code here
}

void CFinalFixDialog::OnBnClickedCheckExcludeDc10()
{
	// TODO: Add your control notification handler code here
}

void CFinalFixDialog::OnBnClickedCheckExcludeDcBlink()
{
	// TODO: Add your control notification handler code here
}

void CFinalFixDialog::OnBnClickedCheckExcludeLongSacc()
{
	// TODO: Add your control notification handler code here
}
