/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

#include "SaccadeEvent.h"

class FixationEvent : public EyeEvent {
public:
	FixationEvent();
	FixationEvent(bool rightEye, bool trackLoss, bool boundary, 
		          unsigned int start, unsigned int end, 
				  short x, short y);
	FixationEvent(const FixationEvent &fix);
	virtual ~FixationEvent();

	const FixationEvent &operator=(const FixationEvent &rhs);

	SaccadeEvent *leadSaccade() const;
	SaccadeEvent *trailSaccade() const;

	const CPoint &charLocation() const;
	const CPoint &pixelLocation() const;

	CRect rectLocation() const;
	
	short index() const;

	void mergeWithFixation(bool mergeWithPrev);
	void mergeWithSaccade();
	void testBlink();
	void update(const CPoint &rgnLoc);
	void updateInitial(const CPoint &rgnLoc);
	void update(int deltaX, int deltaY);

	static  void initialize(bool initRight);

	virtual void draw(CDC *pDC, const Colors &colors, const StateFlags &flags);
	virtual void export(std::ostream &os) const;
	virtual void read(std::istream &is, int versionNum);
	virtual void revert();
	virtual void write(std::ostream &os) const;

protected:
	static short _leftIndex;
	static short _rightIndex;

	struct FixData {
		FixData();
		const FixData &operator=(const FixData &rhs);

		CPoint charLoc;
		CPoint pixelLoc;
		CRect  rectLoc;
		short  index;
	};

	FixData _fd;
	FixData _fdBackup;
};
