/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeDoctor.h"
#include "FinalFixDialog.h"
#include "MainFrm.h"
#include "PreferencesDialog.h"
#include "SummaryDialog.h"
#include <sstream>
#include <iostream>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	// Global help commands
	ON_COMMAND(ID_HELP_FINDER, CFrameWnd::OnHelpFinder)
	ON_COMMAND(ID_HELP, CFrameWnd::OnHelp)
	ON_COMMAND(ID_CONTEXT_HELP, CFrameWnd::OnContextHelp)
	ON_COMMAND(ID_DEFAULT_HELP, CFrameWnd::OnHelpFinder)
	ON_COMMAND(ID_EDIT_PREFERENCES, OnMenuEditPreferences)
	ON_COMMAND(ID_FILE_CLOSE32814, OnMenuFileClose)
	ON_COMMAND(ID_FILE_EXPORT_EXPT_SUMMARY, OnMenuFileExportSummary)
	ON_COMMAND(ID_FILE_EXPORT_EYEDRY, OnMenuFileExportEyeDry)
	ON_COMMAND(ID_FILE_IMPORT, OnMenuFileImport)
	ON_COMMAND(ID_FILE_OPEN32771, OnMenuFileOpen)
	ON_COMMAND(ID_FILE_REVERT_EXPERIMENT, OnMenuFileRevertExperiment)
	ON_COMMAND(ID_FILE_REVERT_TRIAL, OnMenuFileRevertTrial)
	ON_COMMAND(ID_FILE_SAVE32772, OnMenuFileSave)
	ON_COMMAND(ID_FILE_SAVEAS, OnMenuFileSaveAs)
	ON_COMMAND(ID_VIEW_EVENT_FIXATION, OnMenuViewEventFixation)
	ON_COMMAND(ID_VIEW_EVENT_SACCADE, OnMenuViewEventSaccade)
	ON_COMMAND(ID_MENU_VIEW_TRIALDATASUMMARY, OnMenuViewTrialDataSummary)
	ON_COMMAND(ID_MENU_VIEW_EXPERIMENTDATASUMMARY, OnMenuViewExperimentDataSummary)
	ON_COMMAND(ID_TOOLBAR_DELETE_FIX, OnToolbarDeleteFix)
	ON_COMMAND(ID_TOOLBAR_DELETE_TRIAL, OnToolbarDeleteTrial)
	ON_COMMAND(ID_TOOLBAR_FIXATION_DOWN, OnToolbarFixationDown)
	ON_COMMAND(ID_TOOLBAR_FIXATION_LEFT, OnToolbarFixationLeft)
	ON_COMMAND(ID_TOOLBAR_FIXATION_RIGHT, OnToolbarFixationRight)
	ON_COMMAND(ID_TOOLBAR_FIXATION_UP, OnToolbarFixationUp)
	ON_COMMAND(ID_TOOLBAR_IGNORE_FIX, OnToolbarIgnoreFix)
	ON_COMMAND(ID_TOOLBAR_MERGE_NEXT, OnToolbarMergeNext)
	ON_COMMAND(ID_TOOLBAR_MERGE_PREV, OnToolbarMergePrev)
	ON_COMMAND(ID_TOOLBAR_NEXT_TRIAL, OnToolbarNextTrial)
	ON_COMMAND(ID_TOOLBAR_PREVIOUS_TRIAL, OnToolbarPrevTrial)
	ON_COMMAND(ID_TOOLBAR_TRUNCATE_TRIAL, OnToolbarTruncateTrial)
	ON_COMMAND(ID_VIEW_EYE_LEFT, OnMenuViewEyeLeft)
	ON_COMMAND(ID_VIEW_EYE_RIGHT, OnMenuViewEyeRight)
	ON_COMMAND(ID_VIEW_HIGHLIGHT_BLINK, OnMenuViewHighlightBlink)
	ON_COMMAND(ID_VIEW_HIGHLIGHT_DELETED_FIXATIONS, OnMenuViewHighlightDeletedFixations)
	ON_COMMAND(ID_VIEW_HIGHLIGHT_BAD_FIXATIONS, OnMenuViewHighlightFixations)
	ON_COMMAND(ID_VIEW_HIGHLIGHT_BAD_SACCADES, OnMenuViewHighlightSaccades)
	ON_COMMAND(ID_VIEW_STIMULUS_REGIONS, OnMenuViewStimulusRegions)
	ON_COMMAND(ID_VIEW_STIMULUS_TEXT, OnMenuViewStimulusText)
	ON_WM_CLOSE()
	ON_WM_KEYDOWN()
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

CMainFrame::CMainFrame()
  : _dataView(0), _doctorView(0), _trackerView(0), _buttonsEnabled(false)
{}

CMainFrame::~CMainFrame()
{}

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}
#endif //_DEBUG

void CMainFrame::OnClose()
{
	if(_selection.hasExperiment()) {
		int result = MessageBox("Save before closing?", "Save Changes?", MB_YESNOCANCEL | MB_ICONQUESTION);

		if(result == IDYES) 
			OnMenuFileSave();
		else if(result == IDCANCEL) 
			return;
	}

	_selection.clear();
	CFrameWnd::OnClose();
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP | 
		CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) 
		|| !m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create

	}
	// TODO: Delete these three lines if you don't want the toolbar to be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);
	
	CToolBarCtrl &ctrl = m_wndToolBar.GetToolBarCtrl();
	for(int i=ID_TOOLBAR_PREVIOUS_TRIAL; i <= ID_TOOLBAR_IGNORE_FIX; ++i)
		ctrl.SetState(i, TBSTATE_INDETERMINATE);

	return 0;
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// let the view have first crack at the command
	if (m_wndSplit1.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// otherwise, do default handling
	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
{
	BOOL bCreateSplit = m_wndSplit1.CreateStatic(this, 2, 1);

	bCreateSplit &= m_wndSplit2.CreateStatic(&m_wndSplit1, 1, 2, WS_CHILD | WS_VISIBLE | WS_BORDER, m_wndSplit1.IdFromRowCol(0, 0));

	m_wndSplit1.CreateView(1, 0, RUNTIME_CLASS(CDataView), CSize(10, 10), pContext);
	_dataView = static_cast<CDataView*>(m_wndSplit1.GetPane(1, 0));
	_dataView->SetSelectionState(&_selection);

	m_wndSplit2.CreateView(0, 0, RUNTIME_CLASS(CDoctorView), CSize(10, 10), pContext);
	_doctorView = static_cast<CDoctorView*>(m_wndSplit2.GetPane(0, 0));
	_doctorView->SetSelectionState(&_selection);

	m_wndSplit2.CreateView(0, 1, RUNTIME_CLASS(CTrackerView), CSize(800, 600), pContext);
	_trackerView = static_cast<CTrackerView*>(m_wndSplit2.GetPane(0, 1));
	_trackerView->SetSelectionState(&_selection);

	m_wndSplit1.SetRowInfo(0, 500, 10);          //set minimum pane height and width
	m_wndSplit1.SetRowInfo(1, 200, 10);
	m_wndSplit2.SetColumnInfo(0, 210, 10);
	m_wndSplit2.SetColumnInfo(1, 800, 10);

	return (bCreateSplit);
}

void CMainFrame::OnMenuEditPreferences()
{
	CPreferencesDialog dlg(this);

	dlg.Initialize(_selection.colors(), _selection.labelFontHt(), _selection.jhookThreshold());
	if(dlg.DoModal() == IDOK) {
		dlg.GetColors(_selection.colors());
		_selection.setLabelFontHt(dlg.GetFontHeight());
		_selection.setJHookThreshold(dlg.GetJHook());
		_selection.setMultiChange(dlg.GetMultiChange());

		UpdateAllViews();
	}
}

void CMainFrame::OnMenuFileClose()
{
	if(_selection.hasExperiment()) {
		int result = MessageBox("Save before closing?", "Save Changes?", MB_YESNOCANCEL | MB_ICONQUESTION);

		if(result == IDYES) 
			OnMenuFileSave();
		else if(result == IDCANCEL) 
			return;
		_selection.clear();
		UpdateAllViews();
	}
}

void CMainFrame::OnMenuFileExportSummary()
{
	static const char dataFilter[] = "Tab-Delimited Text Files (*.TXT)|*.TXT||";

	if(_selection.hasExperiment()) {
		std::string defaultFName = _selection.filename() + ".TXT";
		CFileDialog fileDlg(FALSE, ".TXT", defaultFName.c_str(), 
							OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, dataFilter, this);

		if(fileDlg.DoModal() == IDOK) {
			const ExperimentData *expt = _selection.experiment();
			CString path = fileDlg.GetPathName();
			std::ofstream ofstrm(path.GetBuffer(), std::ios::out);

			if(ofstrm.is_open() == false) {
				MessageBox("Could not open file for output.", "Export Error");
				return;
			}

			expt->writeSummary(ofstrm, true);
			//expt->writeSummary(std::clog, true);
			ofstrm.close();
		}
	}
}

void CMainFrame::OnMenuFileExportEyeDry()
{
	static const char dataFilter[] = "EyeDry .DA1 Files (*.DA1)|*.DA1||";

	if(_selection.hasExperiment()) {
		CFinalFixDialog finalDlg(this);
		TrialData *trial = _selection.trial();

		finalDlg.SetAvailableEyeData(trial->hasLeftData(), trial->hasRightData());

		if(finalDlg.DoModal() == IDOK) {
			std::string defaultFName = _selection.filename() + ".DA1";
			CFileDialog fileDlg(FALSE, ".DA1", defaultFName.c_str(), 
								OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, dataFilter, this);

			if(fileDlg.DoModal() == IDOK) {
				CSummaryDialog *dlg;
				CString path = fileDlg.GetPathName();
				ExportParams param;
				std::string message;

				param.deleteDist = finalDlg.DeleteFixDist();
				param.deleteLen = finalDlg.DeleteFixLen();
				param.mergeDist = finalDlg.MergeFixDist();
				param.mergeLen = finalDlg.MergeFixLen();
				param.exportDep = finalDlg.ExportDependents();
				param.useRight = finalDlg.UseRightEye();
				param.excludeTens = finalDlg.ExcludeTens() || finalDlg.ExcludeFives() || finalDlg.ExcludeZeros();
				param.excludeFives = finalDlg.ExcludeFives() || finalDlg.ExcludeZeros();
				param.excludeZeros = finalDlg.ExcludeZeros();
				param.exportJHooks = !finalDlg.ExcludeJHooks();
				param.exportBlinkErrors = !finalDlg.ExcludeBlinks();
				param.excludeLongDCSaccs = finalDlg.ExcludeLongSacc();

				try {
					message = _selection.exportExperiment(path.GetBuffer(), param);
				} catch(std::exception &except) {
					MessageBox(except.what(), "Export Error", MB_OK | MB_ICONEXCLAMATION);
					return;
				}

				dlg = new CSummaryDialog();
				dlg->Create(IDD_SUMMARY_DIALOG, this);
				dlg->SetWindowText("Summary of Export Operations");
				dlg->SetDisplayText(message.c_str());
			}
		}
	}
}

void CMainFrame::OnMenuFileImport()
{

	OnMenuEditPreferences();

	static const char dataFilter[] = "Eyelink .ASC Files (*.ASC)|*.asc|DPI .DAT Files (*.DAT)|*.dat||";
	static const char senFilter[] = "DPI .SEN Files (*.SEN)|*.sen||";
	CFileDialog dataDlg(TRUE, ".ASC", NULL, OFN_FILEMUSTEXIST, dataFilter, this);
	CFileDialog senDlg(TRUE, ".SEN", NULL, OFN_FILEMUSTEXIST, senFilter, this);

	if(_selection.hasExperiment()) {
		int result = MessageBox("Close current experiment?", "Close Experiment?", MB_OKCANCEL | MB_ICONQUESTION);

		if(result == IDOK)
			OnMenuFileClose();
		else return;
	}



	if(dataDlg.DoModal() == IDOK) {
		CString dataPath = dataDlg.GetPathName();
		CString ext = dataDlg.GetFileExt();

		if(ext.Compare("DAT") == 0 && senDlg.DoModal() != IDOK)
			return;

		_selection.clear();
		try {
			if(ext.Compare("DAT") == 0) {        //import .DAT file with .SEN file
				CString senPath = senDlg.GetPathName();
				_selection.importExperiment(dataPath.GetBuffer(), senPath.GetBuffer());
			}
			else if(ext.Compare("ASC") == 0 || ext.Compare("asc") == 0)     //import .ASC file
				_selection.importExperiment(dataPath.GetBuffer());
			else throw(std::exception("Unrecognized type for data file.\nMust be one of .DAT or .ASC\n"));
			UpdateAllViews(true);
			} catch(std::exception &except) {
				_selection.clear();
				MessageBox(except.what(), "Error Reading Data", MB_OK | MB_ICONSTOP);
			}
	}

	const ExperimentData *expt = _selection.experiment();
	expt->writeSummary(std::clog, true); // process trial errors w/o writing to a summary file
}

void CMainFrame::OnMenuFileOpen()
{

	static const char dataFilter[] = "Eye Doctor Data Files (*.EDD)|*.edd||";
	CFileDialog dataDlg(TRUE, ".EDD", NULL, OFN_FILEMUSTEXIST, dataFilter, this);

	if(_selection.hasExperiment()) {
		int result = MessageBox("Close current experiment?", "Close Experiment?", MB_OKCANCEL | MB_ICONQUESTION);

		if(result == IDOK)
			OnMenuFileClose();
		else return;
	}

	if(dataDlg.DoModal() == IDOK) {
		CString path = dataDlg.GetPathName();

		try {
			_selection.openExperiment(path.GetBuffer());
			UpdateAllViews(true);
		} catch(std::exception &except) {
			MessageBox(except.what(), "Error Reading Data", MB_OK | MB_ICONSTOP);
		}
	}
}

void CMainFrame::OnMenuFileRevertExperiment()
{
	if(_selection.hasExperiment()) {
		int result = MessageBox("Save experiment changes before reverting?", "Save Changes?", MB_YESNOCANCEL | MB_ICONQUESTION);

		if(result == IDYES)
			OnMenuFileSave();
		else if(result == IDCANCEL)
			return;

		_selection.revertExperiment();
		UpdateAllViews();
	}
}

void CMainFrame::OnMenuFileRevertTrial()
{
	if(_selection.hasExperiment()) {
		TrialData *trial = _selection.trial();
		int result = MessageBox("Save experiment changes before reverting trial?", "Save Changes?", MB_YESNOCANCEL | MB_ICONQUESTION);

		if(result == IDYES)
			OnMenuFileSave();
		else if(result == IDCANCEL)
			return;

		trial->revert();
		UpdateAllViews();
	}
}

void CMainFrame::OnMenuFileSave()
{
	if(_selection.hasExperiment()) {
		if(_selection.filename().empty())
			OnMenuFileSaveAs();
		else _selection.saveExperiment("");
	}
}

void CMainFrame::OnMenuFileSaveAs()
{
	char filterExt[] = "EyeDoctor Data Files (*.EDD)|*.edd||";
	std::string defaultFName = _selection.filename() + ".EDD";
	CFileDialog fileDlg(FALSE, ".EDD", defaultFName.c_str(), 
						OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, filterExt, this);

	if(_selection.hasExperiment() == false) {
		MessageBox("Nothing to save.", "EyeDoctor Warning", MB_OK | MB_ICONWARNING);
		return;
	}

	if(fileDlg.DoModal() == IDOK) {
		try {
		_selection.saveExperiment(fileDlg.GetPathName().GetBuffer());
		} catch(std::exception &except) {
			MessageBox(except.what(), "Error Writing Data", MB_OK | MB_ICONSTOP);
		}
	}
}

void CMainFrame::OnMenuViewEventFixation()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state = flags.test(EVENT_FIXATION) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_EVENT_FIXATION, MF_BYCOMMAND | state);
	if(flags.test(EVENT_FIXATION))
		_selection.setFlag(EVENT_FIXATION, false);
	else _selection.setFlag(EVENT_FIXATION, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewEventSaccade()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state = flags.test(EVENT_SACCADE) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_EVENT_SACCADE, MF_BYCOMMAND | state);
	if(flags.test(EVENT_SACCADE))
		_selection.setFlag(EVENT_SACCADE, false);
	else _selection.setFlag(EVENT_SACCADE, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewExperimentDataSummary()
{
	if(_selection.hasExperiment()) {
		CSummaryDialog *dlg;
		std::ostringstream os;

		os.setf(std::ios::fixed, std::ios::floatfield);
		os.precision(2);
		_selection.experiment()->writeSummary(os, false);

		dlg = new CSummaryDialog();
		dlg->Create(IDD_SUMMARY_DIALOG, this);
		dlg->SetWindowText("Experiment Statistics Summary");
		dlg->SetDisplayText(os.str().c_str());
	}
}

void CMainFrame::OnMenuViewEyeLeft()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state = flags.test(EYE_LEFT) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_EYE_LEFT, MF_BYCOMMAND | state);
	if(flags.test(EYE_LEFT))
		_selection.setFlag(EYE_LEFT, false);
	else _selection.setFlag(EYE_LEFT, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewEyeRight()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state = flags.test(EYE_RIGHT) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_EYE_RIGHT, MF_BYCOMMAND | state);
	if(flags.test(EYE_RIGHT))
		_selection.setFlag(EYE_RIGHT, false);
	else _selection.setFlag(EYE_RIGHT, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewHighlightBlink()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state = flags.test(HIGHLIGHT_BLINK) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_HIGHLIGHT_BLINK, MF_BYCOMMAND | state);
	if(flags.test(HIGHLIGHT_BLINK))
		_selection.setFlag(HIGHLIGHT_BLINK, false);
	else _selection.setFlag(HIGHLIGHT_BLINK, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewHighlightDeletedFixations()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state = flags.test(HIGHLIGHT_DELETED) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_HIGHLIGHT_DELETED_FIXATIONS, MF_BYCOMMAND | state);
	if(flags.test(HIGHLIGHT_DELETED))
		_selection.setFlag(HIGHLIGHT_DELETED, false);
	else _selection.setFlag(HIGHLIGHT_DELETED, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewHighlightFixations()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state = flags.test(HIGHLIGHT_FIXATION) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_HIGHLIGHT_BAD_FIXATIONS, MF_BYCOMMAND | state);
	if(flags.test(HIGHLIGHT_FIXATION))
		_selection.setFlag(HIGHLIGHT_FIXATION, false);
	else _selection.setFlag(HIGHLIGHT_FIXATION, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewHighlightSaccades()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state =flags.test(HIGHLIGHT_SACCADE) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_HIGHLIGHT_BAD_SACCADES, MF_BYCOMMAND | state);
	if(flags.test(HIGHLIGHT_SACCADE))
		_selection.setFlag(HIGHLIGHT_SACCADE, false);
	else _selection.setFlag(HIGHLIGHT_SACCADE, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewStimulusRegions()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state = flags.test(STIMULUS_REGION) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_STIMULUS_REGIONS, MF_BYCOMMAND | state);
	if(flags.test(STIMULUS_REGION))
		_selection.setFlag(STIMULUS_REGION, false);
	else _selection.setFlag(STIMULUS_REGION, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewStimulusText()
{
	const StateFlags &flags = _selection.flags();
	CMenu *menu = GetMenu();
	UINT state = flags.test(STIMULUS_DISPLAY) ? MF_UNCHECKED : MF_CHECKED;

	menu->CheckMenuItem(ID_VIEW_STIMULUS_TEXT, MF_BYCOMMAND | state);
	if(flags.test(STIMULUS_DISPLAY))
		_selection.setFlag(STIMULUS_DISPLAY, false);
	else _selection.setFlag(STIMULUS_DISPLAY, true);
	UpdateAllViews();
}

void CMainFrame::OnMenuViewTrialDataSummary()
{
	if(_selection.hasExperiment()) {
		CSummaryDialog *dlg;
		TrialData *trial = _selection.trial();
		std::ostringstream os;

		os.setf(std::ios::fixed, std::ios::floatfield);
		os.precision(2);
		trial->writeSummary(os, false);

		dlg = new CSummaryDialog();
		dlg->Create(IDD_SUMMARY_DIALOG, this);
		dlg->SetWindowText("Trial Statistics Summary");
		dlg->SetDisplayText(os.str().c_str());
	}
}

void CMainFrame::OnSetFocus(CWnd* /*pOldWnd*/)
{
	m_wndSplit1.SetFocus();
}

void CMainFrame::OnToolbarDeleteFix()
{
	if(_selection.hasExperiment()) {
		FixationEvent *f = _selection.currFixation();
		f->mergeWithSaccade();
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarDeleteTrial()
{
	if(_selection.hasExperiment()) {
		TrialData *t = _selection.trial();
		t->setFlag(TRIAL_DELETED, true);
		theApp.UpdateAllViews();
	}

	// and move to the next one !
	if(_selection.hasExperiment()) {
		_selection.moveTrialPtr(1);
		_selection.initSelectFirst();
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarFixationDown()
{
	if(_selection.hasExperiment()) {
		const SelectionList &selected = _selection.selected();
		TrialData *trial = _selection.trial();

		for(constSelectionIter i = selected.begin(); i != selected.end(); ++i) {
			FixationEvent *event = *i;
			event->update(0, 1);
			event->update(trial->getRegionLocation(event->pixelLocation()));
		}
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarFixationLeft()
{	
	if(_selection.hasExperiment()) {
		const SelectionList &selected = _selection.selected();
		TrialData *trial = _selection.trial();

		for(constSelectionIter i = selected.begin(); i != selected.end(); ++i) {
			FixationEvent *event = *i;
			event->update(-1, 0);
			event->update(trial->getRegionLocation(event->pixelLocation()));
		}
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarFixationRight()
{


	if(_selection.hasExperiment()) {
		const SelectionList &selected = _selection.selected();
		TrialData *trial = _selection.trial();

		for(constSelectionIter i = selected.begin(); i != selected.end(); ++i) {
			FixationEvent *event = *i;
			event->update(1, 0);
			event->update(trial->getRegionLocation(event->pixelLocation()));
		}
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarFixationUp()
{
	if(_selection.hasExperiment()) {
		const SelectionList &selected = _selection.selected();
		TrialData *trial = _selection.trial();

		for(constSelectionIter i = selected.begin(); i != selected.end(); ++i) {
			FixationEvent *event = *i;
			event->update(0, -1);
			event->update(trial->getRegionLocation(event->pixelLocation()));
		}
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarIgnoreFix()
{
	if(_selection.hasExperiment()) {
		FixationEvent *f = _selection.currFixation();
		f->deleteEvent();
		f->update(CPoint(f->charLocation().x, 99));
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarMergeNext()
{
	if(_selection.hasExperiment()) {
		FixationEvent *f = _selection.currFixation();
		f->mergeWithFixation(false);
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarMergePrev()
{
	if(_selection.hasExperiment()) {
		FixationEvent *f = _selection.currFixation();
		f->mergeWithFixation(true);
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarNextTrial()
{
	if(_selection.hasExperiment()) {
		_selection.moveTrialPtr(1);
		_selection.initSelectFirst();
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarPrevTrial()
{
	if(_selection.hasExperiment()) {
		_selection.moveTrialPtr(-1);
		if(_selection.trial()->hasLeftData() || _selection.trial()->hasRightData())
			_selection.initSelectFirst();
		theApp.UpdateAllViews();
	}
}

void CMainFrame::OnToolbarTruncateTrial()
{
	if(_selection.hasExperiment()) {
		FixationEvent *f = _selection.currFixation();
		f->truncateFutureEvents();
		theApp.UpdateAllViews();
	}
}

CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = AfxRegisterWndClass(0);
	return TRUE;
}

void CMainFrame::UpdateAllViews(bool updateExperiment)
{
	CToolBarCtrl &ctrl = m_wndToolBar.GetToolBarCtrl();
	TrialData *trial = _selection.trial();
	std::string title("EyeDoctor");
	int state;
	
	state = trial ? TBSTATE_ENABLED : TBSTATE_INDETERMINATE;
	for(int i=ID_TOOLBAR_PREVIOUS_TRIAL; i <= ID_TOOLBAR_TRUNCATE_TRIAL; ++i)
		ctrl.SetState(i, state);

	state = _selection.selected().empty() ? TBSTATE_INDETERMINATE : TBSTATE_ENABLED;
	for(int i=ID_TOOLBAR_FIXATION_DOWN; i <= ID_TOOLBAR_IGNORE_FIX; ++i)
		ctrl.SetState(i, state);

	if(updateExperiment) {
		const ExperimentData *expt = _selection.experiment();
		const CRect &rect = expt->displaySize();

		_trackerView->SetDisplaySize(rect.Width(), rect.Height());
 		_doctorView->Initialize(rect.Width(), rect.Height(), expt->nTrials(), trial->duration());
	}

	_dataView->UpdateView();
	_doctorView->UpdateView();
	_trackerView->UpdateView();

	if(trial) {
		title.append(": " + _selection.filename());
		title.append(" - ");
		title.append(trial->trialID().idString());

		if(trial->testFlag(TRIAL_DELETED))
			title.append(" (DELETED)");
		else if(trial->testFlag(TRIAL_REPEATED))
			title.append(" (REPEATED)");
		else if(trial->testFlag(TRIAL_ABORTED))
			title.append(" (ABORTED)");
		else if(trial->testFlag(TRIAL_ERROR))
			title.append(" (ERROR)");
		else if(trial->testFlag(TRIAL_TIMEOUT))
			title.append(" (TIMED OUT)");
		else title.append(" (OKAY)");
	}
	SetWindowText(title.c_str());
}

void CMainFrame::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{/*
	if(_selection.hasExperiment()) {
		const SelectionList &selected = _selection.selected();
		TrialData *trial = _selection.trial();
		_selection.initSelectFirst();
		bool update = true;
	}*/

}