/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

#include "DataView.h"
#include "DoctorView.h"
#include "ExperimentData.h"
#include "SelectionState.h"
#include "TrackerView.h"

class CMainFrame : public CFrameWnd
{
public:
	CMainFrame();
	virtual ~CMainFrame();

	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo);

	afx_msg void OnMenuEditPreferences();
	afx_msg void OnMenuFileClose();
	afx_msg void OnMenuFileExportSummary();
	afx_msg void OnMenuFileExportEyeDry();
	afx_msg void OnMenuFileImport();
	afx_msg void OnMenuFileOpen();
	afx_msg void OnMenuFileRevertExperiment();
	afx_msg void OnMenuFileRevertTrial();
	afx_msg void OnMenuFileSave();
	afx_msg void OnMenuFileSaveAs();
	afx_msg void OnMenuViewEventFixation();
	afx_msg void OnMenuViewEventSaccade();
	afx_msg void OnMenuViewExperimentDataSummary();
	afx_msg void OnMenuViewEyeLeft();
	afx_msg void OnMenuViewEyeRight();
	afx_msg void OnMenuViewHighlightBlink();
	afx_msg void OnMenuViewHighlightDeletedFixations();
	afx_msg void OnMenuViewHighlightFixations();
	afx_msg void OnMenuViewHighlightSaccades();
	afx_msg void OnMenuViewStimulusRegions();
	afx_msg void OnMenuViewStimulusText();
	afx_msg void OnMenuViewTrialDataSummary();
	afx_msg void OnToolbarDeleteFix();
	afx_msg void OnToolbarDeleteTrial();
	afx_msg void OnToolbarFixationDown();
	afx_msg void OnToolbarFixationLeft();
	afx_msg void OnToolbarFixationRight();
	afx_msg void OnToolbarFixationUp();
	afx_msg void OnToolbarIgnoreFix();
	afx_msg void OnToolbarMergeNext();
	afx_msg void OnToolbarMergePrev();
	afx_msg void OnToolbarNextTrial();
	afx_msg void OnToolbarPrevTrial();
	afx_msg void OnToolbarTruncateTrial();
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);

	void UpdateAllViews(bool updateExperiment = false);

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected: 
	DECLARE_DYNAMIC(CMainFrame)
	DECLARE_MESSAGE_MAP()

	virtual BOOL OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext);

	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);

	afx_msg void OnClose();
	afx_msg void OnSetFocus(CWnd *pOldWnd);

protected:
	CSplitterWnd  m_wndSplit1;
	CSplitterWnd  m_wndSplit2;
	CStatusBar    m_wndStatusBar;
	CToolBar      m_wndToolBar;

	CDataView      *_dataView;
	CDoctorView    *_doctorView;
	CTrackerView   *_trackerView;
	SelectionState _selection;
	bool           _buttonsEnabled;

};
