/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeDoctor.h"
#include "PreferencesDialog.h"
#include ".\preferencesdialog.h"


// CPreferencesDialog dialog

IMPLEMENT_DYNAMIC(CPreferencesDialog, CDialog)
CPreferencesDialog::CPreferencesDialog(CWnd* pParent /*=NULL*/)
: CDialog(CPreferencesDialog::IDD, pParent), _fontHt(-1){
	for(int i=0; i < _nColors; ++i)
		_color[i] = 0x000000;
}

CPreferencesDialog::~CPreferencesDialog()
{
}

void CPreferencesDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_COLOR_FIELDS, m_listColors);
	DDX_Control(pDX, IDC_STATIC_CURRENT_COLOR, m_staticColor);
	DDX_Control(pDX, IDC_STATIC_PREVIEW, m_staticPreview);
	DDX_Control(pDX, IDC_EDIT_LABEL_FONT_SIZE, m_editFontHt);
}


BEGIN_MESSAGE_MAP(CPreferencesDialog, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_CHANGE_COLOR, OnBnClickedChangeColor)
	ON_LBN_SELCHANGE(IDC_LIST_COLOR_FIELDS, OnLbnSelchangeColorFields)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_EN_CHANGE(IDC_EDIT_LABEL_FONT_SIZE, OnEnChangeLabelFontSize)
	ON_EN_CHANGE(IDC_EDIT_JHOOK_THRESHOLD, OnEnChangeEditJhookThreshold)
	ON_BN_CLICKED(IDC_CHECK_USE_MULTI_CHANGE, OnBnClickedCheckUseMultiChange)
END_MESSAGE_MAP()


// CPreferencesDialog message handlers

void CPreferencesDialog::GetColors(Colors &colors) const
{
	colors.background = _color[0];
	colors.label = _color[1];
	colors.leftEye = _color[2];
	colors.leftEyeHlt = _color[3];
	colors.rightEye = _color[4];
	colors.rightEyeHlt = _color[5];
	colors.selected = _color[6];
	colors.region = _color[7];
	colors.stimulus = _color[8];
	colors.postDisplayChange = _color[9];
}

int CPreferencesDialog::GetFontHeight() const
{
	return(_fontHt);
}

int CPreferencesDialog::GetJHook() const
{
	return(_jhookThreshold);
}

bool CPreferencesDialog::GetMultiChange() const
{
	return(_useMultiChange);
}

void CPreferencesDialog::Initialize(const Colors &colors, int fontHt, int jhookThreshold)
{
	_useMultiChange = false;
	_color[0] = colors.background;
	_color[1] = colors.label;
	_color[2] = colors.leftEye;
	_color[3] = colors.leftEyeHlt;
	_color[4] = colors.rightEye;
	_color[5] = colors.rightEyeHlt;
	_color[6] = colors.selected;
	_color[7] = colors.region;
	_color[8] = colors.stimulus;
	_color[9] = colors.postDisplayChange;

	_fontHt = fontHt;
	_jhookThreshold = jhookThreshold;

}

void CPreferencesDialog::OnBnClickedChangeColor()
{
	const int sel = m_listColors.GetCurSel();
	CColorDialog dlg(_color[sel], CC_ANYCOLOR, this);

	if(dlg.DoModal() == IDOK) {
		_color[sel] = dlg.GetColor();
		RedrawWindow();
	}
}

void CPreferencesDialog::OnBnClickedOk()
{
	if(_fontHt >= 0) {
		char buf[20];

		m_editFontHt.GetWindowText(buf, 20);
		_fontHt = atoi(buf);

		if(_fontHt > 24) {
			MessageBox("Font height must be less than 24 points.", "Invalid Font Height");
			return;
		}
	}

	OnOK();
}

void CPreferencesDialog::OnEnChangeLabelFontSize()
{
	if(_fontHt >= 0) {
		char buf[20];
		int newHt;

		m_editFontHt.GetWindowText(buf, 20);
		newHt = atoi(buf);

		if(newHt > 24) {
			MessageBox("Font height must be less than 24 points.", "Invalid Font Height");
			return;
		}
		else _fontHt = newHt;
	}
	RedrawWindow();
}

BOOL CPreferencesDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	m_listColors.AddString("Background");
	m_listColors.AddString("Fixation Label Text");
	m_listColors.AddString("Left Eye Event");
	m_listColors.AddString("Left Eye Highlight");
	m_listColors.AddString("Right Eye Event");
	m_listColors.AddString("Right Eye Highlight");
	m_listColors.AddString("Selected Event");
	m_listColors.AddString("Stimulus Region Border");
	m_listColors.AddString("Stimulus Text");
	m_listColors.AddString("Post Disp. Change Fix Event");

	m_listColors.SetSel(0);

	if(_fontHt < 0)
		m_editFontHt.EnableWindow(FALSE);
	else {
		char buf[20];
		m_editFontHt.SetWindowText(_itoa(_fontHt, buf, 10));
	}

	char buf[10];
	itoa(_jhookThreshold, buf, 10);
	GetDlgItem(IDC_EDIT_JHOOK_THRESHOLD)->SetWindowText(buf);
	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CPreferencesDialog::OnLbnSelchangeColorFields()
{
	RedrawWindow();
}

void CPreferencesDialog::OnPaint()
{
	CPaintDC dc(this);

	UpdateColorSelection();
	UpdatePreview();
}

void CPreferencesDialog::UpdateColorSelection()
{
	const int selection = m_listColors.GetCurSel();
	CPaintDC dc(&m_staticColor);
	CBrush brush(_color[selection]);
	CRect rect;
	
	m_staticColor.GetClientRect(&rect);
	rect.InflateRect(1, 1);
	dc.FillRect(&rect, &brush);
}

void CPreferencesDialog::UpdatePreview()
{	
	const int nChars = 12;
	const int yOffset = 15;
	const int fontHt = (_fontHt < 0) ? 80 : _fontHt*10;
	CPaintDC dc(&m_staticPreview);
	CBrush bkgBrush(_color[0]);
	CBrush rgnBrush(_color[7]);
	CFont labelFont, stmFont, *oldFont;
	CPen leftPen(PS_SOLID, 1, _color[2]);
	CPen rightPen(PS_SOLID, 1, _color[4]), *oldPen;
	CRect previewRect, region;
	int xOffset;
	
	m_staticPreview.GetClientRect(&previewRect); //fill the background
	previewRect.InflateRect(1, 1);
	dc.FillRect(&previewRect, &bkgBrush);
                                                 //draw stimulus
	dc.SetBkColor(_color[0]);
	dc.SetTextColor(_color[8]);
	stmFont.CreatePointFont(240, "Courier New", &dc);
	oldFont = dc.SelectObject(&stmFont);
	region.CopyRect(previewRect);
	previewRect.MoveToY(previewRect.top+yOffset);
	dc.DrawText("Sample text.", nChars, region, DT_CENTER | DT_NOCLIP | DT_CALCRECT);
	dc.DrawText("Sample text.", nChars, previewRect, DT_CENTER | DT_NOCLIP);
	previewRect.MoveToY(previewRect.top-yOffset);

                                                 //compute and draw individual regions
	xOffset = (previewRect.Width() - region.Width()) / 2;
	region.right /= nChars;
	region.MoveToY(previewRect.top+yOffset);
	for(int i=0; i < nChars; ++i) {
		region.MoveToX(xOffset + i*region.Width());
		dc.FrameRect(&region, &rgnBrush);
	}
                                                 //draw the fixations and saccades
	labelFont.CreatePointFont(fontHt, "Courier New", &dc);
	dc.SelectObject(&labelFont);
	dc.SetBkMode(OPAQUE);
	oldPen = dc.SelectObject(&leftPen);          //left saccade
	dc.MoveTo(20, 20);
	dc.LineTo(180, 30);
	dc.SetBkColor(_color[2]);                    //left fixation
	dc.SetTextColor(_color[1]);
	dc.TextOut(20, 20, "1", 1);
	dc.SetBkColor(_color[3]);                    //left fixation highlight
	dc.TextOut(180, 30, "2", 1);
	dc.SelectObject(&rightPen);                   //right saccade
	dc.MoveTo(50, 60);
	dc.LineTo(220, 45);
	dc.SetBkColor(_color[4]);                    //right fixation
	dc.TextOut(50, 60, "1", 1);
	dc.SetBkColor(_color[5]);                    //right fixation highlight
	dc.TextOut(220, 45, "2", 1);
	dc.SetBkColor(_color[6]);                    //selected fixation
	dc.TextOut(25, 45, "10", 2);

	dc.SelectObject(oldFont);
	dc.SelectObject(oldPen);
}

void CPreferencesDialog::OnEnChangeEditJhookThreshold()
{
	char buf[10];
	GetDlgItem(IDC_EDIT_JHOOK_THRESHOLD)->GetWindowText(buf,10);
	_jhookThreshold = atoi(buf);

}


void CPreferencesDialog::OnBnClickedCheckUseMultiChange()
{
	int state = ((CButton*)GetDlgItem(IDC_CHECK_USE_MULTI_CHANGE))->GetCheck();
	_useMultiChange = (state == BST_CHECKED);
}
