/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#include "EyeEvent.h"

class SaccadeEvent : public EyeEvent {
public:
	SaccadeEvent();
	SaccadeEvent(bool rightEye, bool trackLoss, bool boundary,
				 unsigned int startTime, unsigned int endTime, 
				 short x1, short y1, short x2, short y2);
	SaccadeEvent(const SaccadeEvent &event);
	virtual ~SaccadeEvent();

	const SaccadeEvent &operator=(const SaccadeEvent &rhs);

	CPoint endPixel() const;
	CPoint startPixel() const;

	void update(int deltaX, int deltaY, bool applyToStartPixel);

	virtual void draw(CDC *pDC, const Colors &colors, const StateFlags &flags);
	virtual void export(std::ostream &os) const;
	virtual void read(std::istream &is, int versionNum);
	virtual void revert();
	virtual void write(std::ostream &os) const;

protected:

	struct SacData {
		SacData();
		const SacData &operator=(const SacData &rhs);

        CPoint startPixel;
		CPoint endPixel;
	};

	SacData _sd;
	SacData _sdBackup;
};
