/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#pragma warning( disable : 4290 )

#include <list>
#include "Selections.h"

class ExperimentData;                            //pre-declarations of data classes
class FixationEvent;
class TrialData;

typedef std::list<FixationEvent*>                 SelectionList;
typedef std::list<FixationEvent*>::iterator       SelectionIter;
typedef std::list<FixationEvent*>::const_iterator constSelectionIter;

class SelectionState {
public:
	SelectionState();
	~SelectionState();

	Colors &colors();

	const ExperimentData *experiment() const;

	FixationEvent *currFixation();
	FixationEvent *nextFixation(bool multiSelect);
	FixationEvent *prevFixation(bool multiSelect);

	const SelectionList &selected() const;

	const StateFlags &flags() const;

	TrialData *trial();

	std::string exportExperiment(const std::string &filename, 
								 const ExportParams &params) const throw(std::exception);

	const std::string &filename() const;

	int labelFontHt() const;
	int jhookThreshold() const;
	bool useMultiChange() const;

	bool hasExperiment() const;

	void clear();
	void moveTrialPtr(short delta);
	void importExperiment(const std::string &filename);
	void importExperiment(const std::string &datafile, const std::string &senfile);
	void openExperiment(const std::string &filename) throw(std::exception);
	void revertExperiment();
	void saveExperiment(const std::string &filename);
	void selectFixation(const CPoint &clickPt, bool multiSelect);
	void initSelectFirst(void);
	void setLabelFontHt(int ht);
	void setJHookThreshold(int jh);
	void setMultiChange(bool mc);
	void setFlag(int flag, bool state);

private:
	Colors          _colors;
	ExperimentData *_experiment;
	SelectionList   _selected;
	StateFlags      _flags;
	std::string     _filename;
	int             _labelFontHt;
	int				_jhookThreshold;
	bool			_useMultiChange;
	short           _trialPtr;
};
