/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

#include <bitset>

enum StateFlagValues {
	EVENT_FIXATION,
	EVENT_SACCADE,
	EYE_LEFT,
	EYE_RIGHT,
	HIGHLIGHT_BLINK,
	HIGHLIGHT_DELETED,
	HIGHLIGHT_FIXATION,
	HIGHLIGHT_SACCADE,
	SELECTED,
	STIMULUS_DISPLAY,
	STIMULUS_REGION,
	NUM_STATE_FLAGS
};

enum TrialFlagValues {
	TRIAL_ABORTED,
	TRIAL_DELETED,
	TRIAL_ERROR,
	TRIAL_REPEATED,
	TRIAL_TIMEOUT,
	TRIAL_FASTPRIME,
	TRIAL_PREVIEW,
	TRIAL_NO_END_MESSAGE,
	J_HOOK_DISP_CHANGE,
	BLINK_CHANGE_ERROR,
	NUM_TRIAL_FLAGS
};

typedef std::bitset<NUM_STATE_FLAGS> StateFlags;
typedef std::bitset<NUM_TRIAL_FLAGS> TrialFlags;

struct Colors {
	COLORREF background;
	COLORREF label;
	COLORREF leftEye;
	COLORREF leftEyeHlt;
	COLORREF region;
	COLORREF rightEye;
	COLORREF rightEyeHlt;
	COLORREF selected;
	COLORREF stimulus;
	COLORREF postDisplayChange;
};

struct ExportParams {
	unsigned int deleteLen;
	unsigned int mergeLen;
	int deleteDist;
	int mergeDist;
	bool exportDep;
	bool useRight;
	// subselection flags
	bool excludeTens;
	bool excludeFives;
	bool excludeZeros;
	bool exportJHooks;
	bool exportBlinkErrors;
	bool excludeLongDCSaccs;
};
