/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

#include <fstream>
#include <string>
#include "ProgressDialog.h"

class Tokenizer {
public:
	enum EventType { 
		BLINK_END,
		BLINK_START,
		FIXATION,
		MESSAGE,
		SACCADE,
		TRIAL_END,
		UNKNOWN_EVENT
	};

	enum MessageType {
		DISPLAY_SIZE,
		FONT,
		END_BUTTON,
		EXPERIMENT_STATUS,
		GAZE_TARGET_ON,
		QSTN_ANSWER,
		REGION,
		IMAGE,
		SYNC_TIME,
		TRIAL_ID,
		TRIAL_RESULT,
		TRIAL_STATUS,
		TRIGGERED,
		UNKNOWN_MESSAGE,
		DISP_CHANGE_START,
		DISP_CHANGE_END,
		DRIFTCORRECT
	};

public:
	Tokenizer();
	~Tokenizer();

	EventType readEventType();

	MessageType readMessageType();

	std::string readLine();
	std::string readString(int nChars = -1);

	float readFloat();

	unsigned int readUnsignedInt();
	unsigned int lineNumber() const;

	int fileLength() const;
	int filePosition();
	int readInteger();

	short readShortInt();

	char peekChar(bool ignoreWhitespace = true);
	char readChar(bool ignoreWhitespace = true);

	bool isOkay() const;

	void clearFlags();
	void close();
	void open(const std::string &fileName);
	void readWhitespace();

	int getLastX();
	int getLastY();

	bool checkPosition;
	bool gotChangeCoords;

private:
	std::ifstream   _ifstrm;
	unsigned int    _lineno;
	int             _fileLen;
	int				_lastX;
	int				_lastY;
};
