/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "StdAfx.h"
#include "TrackRegion.h"

TrackRegion::TrackRegion()
  : _boundary(0, 0, 0, 0), _location(-1, -1), 
	_index(-1), _isBoundary(false), _text(0)
{}

TrackRegion::TrackRegion(short x1, short y1, short x2, short y2,
						 short index, const char *text)
  : _boundary(x1, y1, x2, y2), _location(-1, -1),
    _index(index), _isBoundary(false), _text(0)
{
	if(text != 0) {
		int len=0;

		while(text[len] != '\0')
			++len;

		_text = new char[len+1];
		for(int i=0; i <= len; ++i)
			_text[i] = text[i];
	};
}

TrackRegion::TrackRegion(const TrackRegion &region)
{
	(*this) = region;
}

TrackRegion::~TrackRegion()
{
	if(_text) delete _text;
}

const CRect &TrackRegion::boundary() const
{
	return(_boundary);
}

void TrackRegion::draw(CDC *pDC, const Colors &colors, const StateFlags &flags)
{
	if(flags.test(STIMULUS_REGION)) {
		CBrush brush(colors.region);
		pDC->FrameRect(_boundary, &brush);
	}

	if(flags.test(STIMULUS_DISPLAY)) {
		pDC->SetBkMode(TRANSPARENT);
		pDC->SetTextColor(colors.stimulus);
		pDC->TextOut(_boundary.left, _boundary.top, _text, strlen(_text));
	}

	if(_isBoundary) {
		CPen *old, pen(PS_SOLID, 3, colors.leftEye); 

		old = pDC->SelectObject(&pen);
		pDC->MoveTo(_boundary.TopLeft());
		pDC->LineTo(_boundary.left, _boundary.bottom);
		pDC->SelectObject(old);
	}
}

short TrackRegion::index() const
{
	return(_index);
}

const CPoint &TrackRegion::location() const
{
	return(_location);
}

const TrackRegion &TrackRegion::operator=(const TrackRegion &rhs)
{
	_boundary   = rhs._boundary;
	_location   = rhs._location;
	_index      = rhs._index;
	_isBoundary = rhs._isBoundary;

	if(_text)
		delete _text;

	if(!rhs._text)
		_text = 0;
	else {
		int len=0;

		while(rhs._text[len] != '\0')
			++len;

		_text = new char[len+1];
		for(int i=0; i <= len; ++i)
			_text[i] = rhs._text[i];
	}

	return(*this);
}

void TrackRegion::read(std::istream &is, int versionNum)
{
	int sz = sizeof(CRect) + sizeof(CPoint) + sizeof(short);
	std::ios::pos_type pos;
	short len = 1;

	if(versionNum > 2)
		sz +=  sizeof(bool);
	is.read(reinterpret_cast<char*>(&_boundary), sz); //read region data

	pos = is.tellg();                            //mark current position in stream
	while(is.get() != '\0') ++len;               //find the end (length) of text string

	is.seekg(pos);                               //rewind to beginning of text
	_text = new char[len];
	is.read(_text, len);                         //read in text
}

void TrackRegion::setAsBoundary()
{
	_isBoundary = true;
}

void TrackRegion::setLocation(const CPoint &loc)
{
	_location = loc;
}

const char *TrackRegion::text() const
{
	return(_text);
}

void TrackRegion::write(std::ostream &os) const
{
	int sz = sizeof(CRect) + sizeof(CPoint) + sizeof(short) + sizeof(bool);
	short len = 0;

	while(_text[len] != '\0') ++len;             //compute length of string
                                                 //write region data
	os.write(reinterpret_cast<const char*>(&_boundary), sz);
	os.write(_text, len+1);
}
