/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

#include <iosfwd>
#include <list>
#include "Selections.h"

class TrackRegion {
public:
	TrackRegion();
	TrackRegion(short x1, short y1, short x2, short y2,
				short index, const char *text);
	TrackRegion(const TrackRegion &region);
	~TrackRegion();

	const TrackRegion &operator=(const TrackRegion &rhs);

	const CRect &boundary() const;

	const CPoint &location() const;

	short index() const;

	const char *text() const;

	void draw(CDC *pDC, const Colors &colors, const StateFlags &flags);
	void read(std::istream &is, int versionNum);
	void setAsBoundary();
	void setLocation(const CPoint &loc);
	void write(std::ostream &os) const;

private:
	CRect  _boundary;
	CPoint _location;
	short  _index;
	bool   _isBoundary;
	char  *_text;
	
};

typedef std::list<TrackRegion*>                  TrackRegionList;
typedef std::list<TrackRegion*>::iterator        TRIter;
typedef std::list<TrackRegion*>::const_iterator  constTRIter;
