/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#pragma warning( disable : 4290 )

#include <vector>
#include "Selections.h"
#include "FixationEvent.h"
#include "TrackRegion.h"
#include "TrialId.h"
#include "Tokenizer.h"

#define DISPLAY_REFRESH_LAG 0

typedef struct imagePos{
	imagePos(std::string fName, CPoint pt);
	imagePos();
	~imagePos();

	std::string fileName;
	CPoint upperLeft;
	const imagePos &operator=(const imagePos &ip);
}IMAGEPOS;

typedef std::vector<IMAGEPOS> ImageVector;

typedef struct dcData{
	dcData(CPoint p, unsigned int t);
	dcData(CPoint p, unsigned int t, unsigned int fixt, unsigned int leadst, unsigned int leadend);
	dcData();
	void read(std::istream &is, int versionNum);
	void write(std::ostream &os) const;
	~dcData();
	

	unsigned int time;
	CPoint loc;
	unsigned int fixTime;
	unsigned int leadStart;
	unsigned int leadEnd;
	//const dcData &operator=(const dcData &dc);
}DCDATA;

typedef std::vector<DCDATA> dcVector;

class TrialData {
public:
	TrialData();
	TrialData(const std::string &trialId);
	~TrialData();

	FixationEvent *selectFixation(const CPoint &clickPt, bool leftEye, bool rightEye);
	FixationEvent *selectFirstFixation(bool leftEye, bool rightEye);

	const TrialId &trialID() const;

	CPoint getRegionLocation(const CPoint &point) const;

	const std::string &fontName() const;

	unsigned int duration() const;

	int fontHeight() const;

	short endButton() const;
	short index() const;
	short questionStatus() const;

	bool hasLeftData() const;
	bool hasRightData() const;
	bool isDependent() const;
	bool testFlag(int flag_id) const;

	void drawEvents(CDC *pDC, const Colors &colors, const StateFlags &flags);
	void drawRegions(CDC *pDC, const Colors &colors, const StateFlags &flags);
	void drawImages(CDC *pDC, const Colors &colors, const StateFlags &flags);
	void export(std::ostream &exportStrm, std::ostream &changeStrm, const ExportParams &param);
	void import(Tokenizer &tok, const int j_hook_size = 10) throw(std::exception);
	void importData(Tokenizer &dataTok, int firstLine) throw(std::exception);
	void importSen(Tokenizer &senTok) throw(std::exception);
	void read(std::istream &is, int versionNum);
	void revert();
	void setFlag(int flag_id, bool value);
	void setIndex(short index);
	void write(std::ostream &os) const;
	void writeSummary(std::ostream &os, bool tabbed)/* const*/;
	void setDisplayCoords(CRect r);
	int dcTime();
	unsigned int getLastDCTime(unsigned int fTime);
	unsigned int getLastLeadStart(unsigned int fTime);
	unsigned int getLastLeadEnd(unsigned int fTime);
	dcData getDCData(unsigned int eTime);
	//dcData readDcData(std::istream &is, int versionNum);
	//void writeDcData(std::ostream &os) const;

private:
	EyeEvent       *_leftEvents;                 //left eye event list
	EyeEvent       *_rightEvents;                //right eye event list
	TrackRegionList _regions;                    //tracking regions
	TrialFlags      _flags;
	TrialId			_trialId;
	ImageVector		_images;
	std::string     _fontName;
	unsigned int    _endTime;					 //trial end time
	unsigned int    _disp2Time;                  //time 2nd display activated (DPI-preview)
	unsigned int    _disp3Time;                  //time 3rd display activated (DPI-fast prime)
	unsigned int    _syncTime;                   //SYNCTIME (same as start time for non-gaze contingent trials)
	int             _fontHt;
	short           _endButton;                  //button used to end trial
	short           _index;
	short           _previewBdry;                //character boundary in preview/fast prime trials
	short           _previewResult;              //result of preview/fast prime trials
	short			_qstnAnswer;
	CRect			_displayCoords;
	CPoint			_dispChangeCoords;
	dcVector		_dcVector;


public:
	bool dcOK;
	bool dcWithin5;
	bool dcWithin10;
	bool hasLongSacc;
	int dcLeadSaccStart;
	int dcLeadSaccEnd;
};

typedef std::vector<TrialData*> TrialVector;



