/*  EyeDoctor software is used to view, edit and translate data generated
    via Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "TrialId.h"
#include <sstream>

TrialId::TrialId()
  : _type(TrialId::FILLER), _condition(0), _item(0), _index(0), _isOld(false)
{}

TrialId::TrialId(const std::string &identifier)
  : _type(TrialId::FILLER), _condition(0), _item(0), _index(0), _isOld(false)
{
	std::istringstream is(identifier);
	char ch;

	ch = is.get();
	switch(ch) {
		case 'c':
		case 'C': _isOld = true;
				  _type = TrialId::EXPERIMENTAL;  break;
		case 'e':
		case 'E': _type = TrialId::EXPERIMENTAL;  break;
		case 'f':
		case 'F': _type = TrialId::FILLER;        break;
		case 'p':
		case 'P': _type = TrialId::PRACTICE;      break;
	}

	is>>_condition;

	ch = is.get();
	is>>_item;

	if(_isOld == false) {
		ch = is.get();
		is>>_index;
	}
}

TrialId::TrialId(const TrialId &id)
{
	_type       = id._type;
	_condition  = id._condition;
	_item       = id._item;
	_index      = id._index;
	_isOld      = id._isOld;
}

TrialId::~TrialId()
{}

int TrialId::compare(const TrialId &id) const
{
	if(_type < id._type)
		return(-1);
	else if(_type > id._type)
		return(1);

	if(_item < id._item) 
		return(-1);
	if(_item == id._item && _condition < id._condition) 
		return(-1);
	if(_item == id._item && _condition == id._condition && _index < id._index)
		return(-1);
	if(_item == id._item && _condition == id._condition && _index == id._index)
		return(0);
	return(1);
}

unsigned short TrialId::condition() const
{
	return(_condition);
}

std::string TrialId::idString() const
{
	std::ostringstream os;

	if(_isOld)
		os<<'C';
	else {
		switch(_type) {
			case TrialId::EXPERIMENTAL: os<<'E';  break;
			case TrialId::FILLER:       os<<'F';  break;
			case TrialId::PRACTICE:     os<<'P';  break;
		}
	}
	
	os<<_condition<<'I'<<_item;
	if(!_isOld)
		os<<'D'<<_index;
	return(os.str());
}

unsigned short TrialId::index() const
{
	return(_index);
}

bool TrialId::isDependent() const
{
	if(_isOld) return(_condition > 99);
	else return(_index > 0);
}

bool TrialId::isExperimental() const
{
	return(_type == TrialId::EXPERIMENTAL);
}

bool TrialId::isFiller() const
{
	return(_type == TrialId::FILLER);
}

bool TrialId::isOld() const
{
	return(_isOld);
}

bool TrialId::isPractice() const
{
	return(_type == TrialId::PRACTICE);
}

unsigned short TrialId::item() const
{
	return(_item);
}

const TrialId &TrialId::operator=(const TrialId &rhs)
{
	_type       = rhs._type;
	_condition  = rhs._condition;
	_item       = rhs._item;
	_index      = rhs._index;
	_isOld      = rhs._isOld;
	return(*this);
}

TrialId::Type TrialId::type() const
{
	return(_type);
}