/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeDoctor.h"
#include "TrialSelectionDialog.h"
#include ".\trialselectiondialog.h"

/********************* CTrialSelectionDialog::Selection **********************/

CTrialSelectionDialog::Selection::Selection()
   : condMax(0), condMin(1), itemMax(0), itemMin(1), depMax(0), depMin(0),
     useExperimental(false), useFiller(false), usePractice(false)
{}

const CTrialSelectionDialog::Selection &
CTrialSelectionDialog::Selection::operator=(const CTrialSelectionDialog::Selection &rhs)
{
	condMax         = rhs.condMax;
	condMin         = rhs.condMin;
	itemMax         = rhs.itemMax;
	itemMin         = rhs.itemMin;
	depMax          = rhs.depMax;
	depMin          = rhs.depMin;
	useExperimental = rhs.useExperimental;
	useFiller       = rhs.useFiller;
	usePractice     = rhs.usePractice;
	return(*this);
}

/*************************** CTrialSelectionDialog ***************************/

IMPLEMENT_DYNAMIC(CTrialSelectionDialog, CDialog)
CTrialSelectionDialog::CTrialSelectionDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CTrialSelectionDialog::IDD, pParent), _selection()
{}

CTrialSelectionDialog::~CTrialSelectionDialog()
{}

void CTrialSelectionDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CTrialSelectionDialog, CDialog)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDC_CHECK_EXPERIMENTAL, OnBnClickedExperimental)
	ON_BN_CLICKED(IDC_CHECK_FILLER, OnBnClickedFiller)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_CHECK_PRACTICE, OnBnClickedPractice)
	ON_EN_KILLFOCUS(IDC_EDIT_DEP_MAX, OnEnKillfocusDepMax)
	ON_EN_KILLFOCUS(IDC_EDIT_DEP_MIN, OnEnKillfocusDepMin)
	ON_EN_KILLFOCUS(IDC_EDIT_COND_MAX, OnEnKillfocusCondMax)
	ON_EN_KILLFOCUS(IDC_EDIT_COND_MIN, OnEnKillfocusCondMin)
	ON_EN_KILLFOCUS(IDC_EDIT_ITEM_MAX, OnEnKillfocusItemMax)
	ON_EN_KILLFOCUS(IDC_EDIT_ITEM_MIN, OnEnKillfocusItemMin)
	ON_BN_CLICKED(IDC_CHECK_DC, OnBnClickedCheckDc)
END_MESSAGE_MAP()

// CTrialSelectionDialog message handlers

const CTrialSelectionDialog::Selection &CTrialSelectionDialog::GetSelections() const
{
	return(_selection);
}

void CTrialSelectionDialog::OnBnClickedCancel()
{
	_selection = Selection();
	OnCancel();
}

void CTrialSelectionDialog::OnBnClickedExperimental()
{
	CButton *btn = (CButton*)GetDlgItem(IDC_CHECK_EXPERIMENTAL);

	if(btn->GetCheck() == BST_CHECKED)
		_selection.useExperimental = true;
	else _selection.useExperimental = false;
}

void CTrialSelectionDialog::OnBnClickedFiller()
{
	CButton *btn = (CButton*)GetDlgItem(IDC_CHECK_FILLER);

	if(btn->GetCheck() == BST_CHECKED)
		_selection.useFiller = true;
	else _selection.useFiller = false;
}

void CTrialSelectionDialog::OnBnClickedOk()
{
	if(_selection.useExperimental == false && 
	   _selection.useFiller == false && 
	   _selection.usePractice == false) {
		   MessageBox("No trials selected. Select at least one\nof Experimental, Filler or Practice.", "Select Trials");
		   return;
	}

	if(_selection.condMax < _selection.condMin) {
		MessageBox("No trials selected. Select a range\nof one or more condition numbers.", "Select Trials");
		return;
	}

	if(_selection.itemMax < _selection.itemMin) {
		MessageBox("No trials selected. Select a range\nof one or more item numbers.", "Select Trials");
		return;
	}

	if(_selection.depMax < _selection.depMin) {
		MessageBox("No trials selected. Select a range\nof one or more dependency numbers.", "Select Trials");
		return;
	}

	OnOK();
}

void CTrialSelectionDialog::OnBnClickedPractice()
{
	CButton *btn = (CButton*)GetDlgItem(IDC_CHECK_PRACTICE);

	if(btn->GetCheck() == BST_CHECKED)
		_selection.usePractice = true;
	else _selection.usePractice = false;
}

void CTrialSelectionDialog::OnEnKillfocusCondMax()
{
	CEdit *edit = (CEdit*)GetDlgItem(IDC_EDIT_COND_MAX);
	CString str;

	edit->GetWindowText(str);
	_selection.condMax = atoi(str.GetBuffer());
}

void CTrialSelectionDialog::OnEnKillfocusCondMin()
{
	CEdit *edit = (CEdit*)GetDlgItem(IDC_EDIT_COND_MIN);
	CString str;

	edit->GetWindowText(str);
	_selection.condMin = atoi(str.GetBuffer());
}

void CTrialSelectionDialog::OnEnKillfocusDepMax()
{
	CEdit *edit = (CEdit*)GetDlgItem(IDC_EDIT_DEP_MAX);
	CString str;

	edit->GetWindowText(str);
	_selection.depMax = atoi(str.GetBuffer());
}

void CTrialSelectionDialog::OnEnKillfocusDepMin()
{
	CEdit *edit = (CEdit*)GetDlgItem(IDC_EDIT_DEP_MIN);
	CString str;

	edit->GetWindowText(str);
	_selection.depMin = atoi(str.GetBuffer());
}

void CTrialSelectionDialog::OnEnKillfocusItemMax()
{
	CEdit *edit = (CEdit*)GetDlgItem(IDC_EDIT_ITEM_MAX);
	CString str;

	edit->GetWindowText(str);
	_selection.itemMax = atoi(str.GetBuffer());
}

void CTrialSelectionDialog::OnEnKillfocusItemMin()
{
	CEdit *edit = (CEdit*)GetDlgItem(IDC_EDIT_ITEM_MIN);
	CString str;

	edit->GetWindowText(str);
	_selection.itemMin = atoi(str.GetBuffer());
}

BOOL CTrialSelectionDialog::OnInitDialog()
{
	CWnd *ctrl;
	char buf[20];

	CDialog::OnInitDialog();

	ctrl = GetDlgItem(IDC_CHECK_EXPERIMENTAL);
	if(_selection.useExperimental)
		((CButton*)ctrl)->SetCheck(BST_CHECKED);
	else ctrl->EnableWindow(FALSE);

	ctrl = GetDlgItem(IDC_CHECK_FILLER);
	if(_selection.useFiller)
		((CButton*)ctrl)->SetCheck(BST_CHECKED);
	else ctrl->EnableWindow(FALSE);

	ctrl = GetDlgItem(IDC_CHECK_PRACTICE);
	if(_selection.usePractice)
		((CButton*)ctrl)->SetCheck(BST_CHECKED);
	else ctrl->EnableWindow(FALSE);

	ctrl = GetDlgItem(IDC_EDIT_COND_MAX);
	ctrl->SetWindowText(_itoa(_selection.condMax, buf, 10));

	ctrl = GetDlgItem(IDC_EDIT_COND_MIN);
	ctrl->SetWindowText(_itoa(_selection.condMin, buf, 10));

	ctrl = GetDlgItem(IDC_EDIT_ITEM_MAX);
	ctrl->SetWindowText(_itoa(_selection.itemMax, buf, 10));

	ctrl = GetDlgItem(IDC_EDIT_ITEM_MIN);
	ctrl->SetWindowText(_itoa(_selection.itemMin, buf, 10));

	if(_selection.depMax > 0) {
		ctrl = GetDlgItem(IDC_EDIT_DEP_MAX);
		ctrl->SetWindowText(_itoa(_selection.depMax, buf, 10));
		ctrl = GetDlgItem(IDC_EDIT_DEP_MIN);
		ctrl->SetWindowText(_itoa(_selection.depMin, buf, 10));
	}
	else {
		ctrl = GetDlgItem(IDC_EDIT_DEP_MAX);
		ctrl->EnableWindow(FALSE);
		ctrl = GetDlgItem(IDC_EDIT_DEP_MIN);
		ctrl->EnableWindow(FALSE);
	}

	return TRUE;
}

void CTrialSelectionDialog::SetSelections(const CTrialSelectionDialog::Selection &selection)
{
	_selection = selection;
}

void CTrialSelectionDialog::OnBnClickedCheckDc()
{
	CButton *btn = (CButton*)GetDlgItem(IDC_CHECK_DC);

	if(btn->GetCheck() == BST_CHECKED)
		_selection.useDCok = true;
	else _selection.useDCok = false;
}
